<?php

namespace kernel\console\controllers;

use kernel\console\ConsoleController;
use kernel\helpers\Files;
use ZipArchive;

class KernelController extends ConsoleController
{
    protected Files $files;

    public function __construct()
    {
        parent::__construct();
        $this->files = new Files();
    }

    /**
     * @throws \Exception
     */
    public function actionPackKernel(): void
    {
        if (!isset($this->argv['path'])) {
            throw new \Exception('Missing kernel path "--path" specified');
        }

        if (file_exists(ROOT_DIR . $this->argv['path'])) {
            $tmpKernelDirFull = RESOURCES_DIR . '/tmp/ad/app/kernel';
            $this->files->copy_folder(KERNEL_DIR, $tmpKernelDirFull);
            $this->out->r("Ядро скопировано во временную папку", 'green');
        } else {
            $this->out->r("Ядро не найдено", 'red');
        }

        if (isset($this->argv['bootstrap'])) {
            if (file_exists(ROOT_DIR . '/bootstrap')) {
                $tmpBootstrapDirFull = RESOURCES_DIR . '/tmp/ad/app/bootstrap';
                $this->files->copy_folder(ROOT_DIR . '/bootstrap', $tmpBootstrapDirFull);
                $this->out->r("/bootstrap скопирован во временную папку", 'green');
            } else {
                $this->out->r("/bootstrap не найден", 'red');
            }
        }

        if (isset($this->argv['.env'])) {
            if (file_exists(ROOT_DIR . '/.env.example')) {
                $tmpEnvFull = RESOURCES_DIR . '/tmp/ad/app/env.example';
               copy(ROOT_DIR . '/.env.example', $tmpEnvFull);
                $this->out->r("/.env.example скопирован во временную папку", 'green');
            } else {
                $this->out->r("/.env.example не найден", 'red');
            }
        }

        if (!is_dir(RESOURCES_DIR . '/tmp/app')) {
            mkdir(RESOURCES_DIR . '/tmp/app');
        }

        $this->files->pack(RESOURCES_DIR . '/tmp/ad/app/', RESOURCES_DIR . '/tmp/app/app.iga');
        $this->files->recursiveRemoveDir(RESOURCES_DIR . '/tmp/ad/app/');
    }

    /**
     * @throws \Exception
     */
    public function actionUpdateKernel(): void
    {
        if (!isset($this->argv['path'])) {
            throw new \Exception('Missing kernel path "--path" specified');
        }

        $zip = new ZipArchive;
        if (file_exists(ROOT_DIR . $this->argv['path'])) {
            $tmpAppDir = md5(time());
            $res = $zip->open(ROOT_DIR . $this->argv['path']);
            if ($res === TRUE) {
                $tmpAppDirFull = RESOURCES_DIR . '/tmp/app/' . $tmpAppDir . "/";
                $zip->extractTo($tmpAppDirFull);
                $zip->close();
                $this->files->recursiveRemoveKernelDir();
                $this->files->copy_folder($tmpAppDirFull . 'kernel' , ROOT_DIR . "/kernel");

                if (isset($this->argv['bootstrap'])) {
                    $this->files->recursiveRemoveDir(ROOT_DIR . '/bootstrap');
                    $this->files->copy_folder($tmpAppDirFull . 'bootstrap' , ROOT_DIR . '/bootstrap');
                }

                if (isset($this->argv['.env'])) {
                    copy($tmpAppDirFull . 'env.example', ROOT_DIR . '/.env.example');
                }

                $this->files->recursiveRemoveDir($tmpAppDirFull);
                $this->out->r('Приложение обновлено.', 'green');
            } else {
                $this->out->r('unable to open zip archive', 'red');
            }
        } else {
            $this->out->r("archive not found", 'red');
        }
    }

}